<?php

namespace Concrete\Package\AbStrToImgTtf\Block\AbStrToImgTtf;

use Concrete\Core\Block\BlockController;

/**
 * ab_str_to_img_ttf package for Concrete5
 * Copyright Copyright 2017-2018, Alex Borisov
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @author Alex Borisov <linuxoidoz@gmail.com>
 * @package Concrete\Package\ab_str_to_img_ttf
 */
 
class Controller extends BlockController
{
    
    protected $btWrapperClass = 'ccm-ui';
    protected $btInterfaceWidth = "400";
    protected $btInterfaceHeight = "500";
    protected $btTable = 'btAbStrToImgTtf';
    protected $btDefaultSet = 'multimedia';
    
    public function getBlockTypeDescription() 
    {
        return t("Add TTF String to Image block");
    }

    public function getBlockTypeName() 
    {
        return t("TTF String to Image");
    }

    public function registerViewAssets($outputContent = '')
    {
        $this->requireAsset('javascript', 'jquery');
    }
    

    public function on_start()
    {
        $this->set('app', $this->app);
    }
    
    public function view() 
    {
        $imgData = $this->make_image();
        $this->set('imgData', $imgData);
    }

    public function add()
    {
        $fonts = $this->get_fonts();
        ksort($fonts);
        $this->set('fonts', array_keys($fonts));
    }
    
    public function edit()
    {
        $fonts = $this->get_fonts();
        ksort($fonts);
        $this->set('fonts', array_keys($fonts));
        if (!empty($fonts) && in_array($this->font, array_keys($fonts))) {
            $this->set('font_selected', array_search($this->font, array_keys($fonts)));
        }
        else {
            $this->set('font_selected', 0);
        }
    }
    
    public function save($args) 
    {
        $fonts = $this->get_fonts();
        ksort($fonts);
        
        $args['img_string'] = isset($args['img_string']) ? trim($args['img_string']) : '';
        $args['color'] = isset($args['color']) ? trim($args['color']) : '#0099ff';
        $args['font'] = (isset($args['font']) && !empty($fonts)) ? array_keys($fonts)[$args['font']] : '';
        $args['size'] = isset($args['size']) ? trim($args['size']) : '12';
        parent::save($args);
        
        $expensiveCache = $this->app->make('cache/expensive');
        $cacheItem = $expensiveCache->getItem('AbStrToImgTtf/img' . $this->bID);
        if (!$cacheItem->isMiss()) {
            $cacheItem->clear();
        }
    }

    public function make_image()
    {
        $imgData = null;
        $fonts = $this->get_fonts();
        if (!empty($fonts) && in_array($this->font, array_keys($fonts))) {
            
            $expensiveCache = $this->app->make('cache/expensive');
            $cacheItem = $expensiveCache->getItem('AbStrToImgTtf/img' . $this->bID);
            if ($cacheItem->isMiss()) {
                $helperFile = $this->app->make('helper/file');
                
                $font_family = pathinfo($fonts[$this->font])['dirname'] . '/' . $this->font . '.ttf';
                $text_box = imagettfbbox($this->size, 0, $font_family, $this->img_string);
                $text_width = abs($text_box[2] - $text_box[0]);
                $text_height = abs($text_box[7] - $text_box[1]);
                $x = 1;
                $y = $text_height * 0.9;

                $img = imagecreatetruecolor($text_width + 3, $text_height*1.2);
                if (!$img) {
                    return $imgData;
                }
                imagesavealpha($img, true);
                Imagealphablending($img, false);
                $trans_color = imagecolorallocatealpha($img, 255, 255, 255, 127);
                imagefilledrectangle($img, 0, 0, $text_width + 3, $text_height*1.2, $trans_color);
                
                $hex = str_replace("#", "", $this->color);
                $color = hexdec($hex);
                $red = 0xFF & ($color >> 0x10);
                $green = 0xFF & ($color >> 0x8);
                $blue = 0xFF & $color;
                $textcolor = imagecolorallocate($img, $red, $green, $blue);
                imagettftext($img, $this->size, 0, $x, $y, $textcolor, $font_family, $this->img_string);
                
                ob_start();
                imagepng($img);
                $imgData = ob_get_clean();
                imagedestroy($img);
                
                $expensiveCache->save($cacheItem->set($imgData));
            }
            else {
                $imgData = $cacheItem->get();
            }
        }
        
        return $imgData;
    }
    
    public function get_fonts() {
        $fonts = array();
        
        $app_fonts_folder = DIR_FILES_UPLOADED_STANDARD . '/fonts/ttf/';
        clearstatcache(true, $app_fonts_folder);
        if (is_dir($app_fonts_folder)) {
            $user_files = glob($app_fonts_folder . '*.ttf');
            foreach ($user_files as $filename => $path) {
                $filename = pathinfo($path)['filename'];
                $fonts[$filename] = $path;
            }
        }
        
        $pkg = \Concrete\Core\Package\Package::getByHandle('ab_str_to_img_ttf');
        $pkg_fonts_folder = $pkg->getPackagePath() . '/blocks/ab_str_to_img_ttf/ttf/';
        $pkg_files = glob($pkg_fonts_folder . '*.ttf');
        foreach ($pkg_files as $filename => $path) {
            $filename = pathinfo($path)['filename'];
            $fonts[$filename] = $path;
        }
        
        return $fonts;
    }
     
}
